# Seen - track when a person speaks, and allow data to be queried
# Copyright (C) 2010  Brian S. Stephan
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ConfigParser import NoOptionError

from irclib import irclib
from datetime import datetime
from dateutil.tz import *

from Module import Module

# Keeps track of when people say things in public channels, and reports on when
# they last said something.

class Seen(Module):

    def __init__(self, config, server, modlist, rehash):
        super(Seen, self).__init__(config, server, modlist, rehash)
        modlist.append(self)

    def register_handlers(self, server):
        server.add_global_handler('pubmsg', self.on_pubmsg)
        server.add_global_handler('privmsg', self.on_privmsg)

    def do(self, connection, event, nick, userhost, replypath, what, admin_unlocked):
        # whatever it is, store it
        if not self.config.has_section('seen'):
            self.config.add_section('seen')

        self.config.set('seen', nick, userhost + '|:|' + datetime.utcnow().isoformat() + '|:|' + what)

        # also see if it's a query
        whats = what.split(' ')
        if whats[0] == 'seen' and len(whats) >= 2:
            query = whats[1]
            if query != 'debug':
                try:
                    seendata = self.config.get('seen', query).split('|:|')
                    converted = datetime.strptime(seendata[1], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=tzutc())
                    replystr = 'last saw ' + query + ' at ' + converted.astimezone(tzlocal()).strftime("%Y/%m/%d %H:%M:%S %Z") + ' saying \'' + seendata[2] + '\''
                    if replypath is None:
                        return replystr
                    else:
                        connection.privmsg(replypath, replystr)
                except NoOptionError: pass

# vi:tabstop=4:expandtab:autoindent
# kate: indent-mode python;indent-width 4;replace-tabs on;
